/*
**    Copyright 2009 Sogame
**
**    This file is part of Define.
**
**    Define is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    Define is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with Define. If not, see <http://www.gnu.org/licenses/>.
*/

var define_extension = {
	textMaxLength: 15,
	define_url: "http://www.google.com/search?ie=utf-8&oe=utf-8&q=define%3A",
	is_browser: true,
	context_menu_id: "contentAreaContextMenu",
	
	
	define: function() {
		var url = this.define_url;
		var elem;
		var selected = gContextMenu.isTextSelected;
		var text = "";
		var settings = this.loadSettings();
		
		//if (selected) {
		if (selected || (selected == undefined)) { // Thunderbird 3.0
			var cur_win = document.commandDispatcher.focusedWindow;
			text = cur_win.getSelection();
			text = text.toString();
			text = define_extension.trim(text);
		}
		else if (gContextMenu.onTextInput) {
			var input = gContextMenu.target;
			if (input.selectionStart != undefined) {  
				text = input.value.substr(input.selectionStart, input.selectionEnd - input.selectionStart);  
				text = text.toString();
				text = define_extension.trim(text);
			}		
		}
		url += encodeURI(this.replaceAll(text, " ", "+"));
		
		if (define_extension.is_browser) {
			switch (settings["target"]) {
				case 0:
					// Current tab
					var brwsr = gBrowser.getBrowserForTab(gBrowser.mCurrentTab);
					brwsr.loadURI(url);
					break;
				case 2:
					// New window
					window.open(url);
					break;
				default:
					// New tab
					var pos = -1;
					try {
						pos = gBrowser.tabContainer.selectedIndex + 1;
					}
					catch (e) {
						pos = -1;
					}
					var new_tab = gBrowser.addTab(url);
					if (settings["tab_nextto"] && (pos >= 0)) {
						try {
							gBrowser.moveTabTo(new_tab, pos);
						}
						catch (e) { }
					}
					if (settings["focus_tab"]) {
						gBrowser.selectedTab = new_tab;
					}
					break;
			}
		}
		else {
			var uri = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService).newURI(url, null, null);
			var com = Components.classes["@mozilla.org/uriloader/external-helper-app-service;1"];
			var httpHandler = com.createInstance(Components.interfaces.nsIExternalProtocolService);
			httpHandler.loadUrl(uri);
		}
	},
	
	
	updateVendorInfo: function() {
		var app_info = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);
		if (app_info.ID == "{3550f703-e582-4d05-9a08-453d09bdfdc6}") {
			// Thunderbird
			define_extension.is_browser = false;
			define_extension.context_menu_id = "mailContext";
		}
	},
	
	
	showContextMenuEvent: function() {
		window.removeEventListener("load", define_extension.showContextMenuEvent, true);
		
		define_extension.updateVendorInfo();
		
		var elem = document.getElementById(define_extension.context_menu_id);
		if (elem != null) {
			elem.addEventListener("popupshowing", define_extension.showContextMenuItems, false);
		}
	},
	
	
	showContextMenuItems: function() {
		var elem;
		var selected = gContextMenu.isTextSelected;
		var hidden = true;
		var text = "";
		
		//if (selected) {
		if (selected || (selected == undefined)) { // Thunderbird 3.0
			var cur_win = document.commandDispatcher.focusedWindow;
			text = cur_win.getSelection();
			text = text.toString();
			text = define_extension.trim(text);
			hidden = (text == "");
		}
		else if (gContextMenu.onTextInput) {
			var input = gContextMenu.target;
			if (input.selectionStart != undefined) {  
				text = input.value.substr(input.selectionStart, input.selectionEnd - input.selectionStart);  
				text = text.toString();
				text = define_extension.trim(text);
				hidden = (text == "");
			}		
		}
		
		if (text.length > define_extension.textMaxLength) {
			text = text.substr(0, define_extension.textMaxLength) + '...';
		}
		
		elem = document.getElementById("define_context_menu_op");
		elem.hidden = hidden;
		elem.label = document.getElementById("bundle_define").getString("define_str") + ' "' + text + '"';
	},
	
	
	trim: function(s) {
		while (s.substring(0, 1) == ' ') {
			s = s.substring(1, s.length);
		}
		while (s.substring(s.length-1, s.length) == ' ') {
			s = s.substring(0, s.length-1);
		}
		return s;
	},
	
	
	replaceAll: function(streng, soeg, erstat) {
		var st = streng;
		if (soeg.length == 0)
			return st;
		var idx = st.indexOf(soeg);
		while (idx >= 0) {
			st = st.substring(0, idx) + erstat + st.substr(idx + soeg.length);
			idx = st.indexOf(soeg);
		}
		return st;
	},
	
	
	//
	// Options
	//
	
	loadSettings: function() {
		var arr = Array();
		try {
			var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
			if (prefs.prefHasUserValue("define.target")) {
				arr["target"] = prefs.getIntPref("define.target");
			}
			else {
				prefs.setIntPref("define.target", 1);
				arr["target"] = 1;
			}
			
			if (prefs.prefHasUserValue("define.focus_tab")) {
				arr["focus_tab"] = prefs.getBoolPref("define.focus_tab");
			}
			else {
				prefs.setBoolPref("define.focus_tab", true);
				arr["focus_tab"] = true;
			}
			
			if (prefs.prefHasUserValue("define.tab_nextto")) {
				arr["tab_nextto"] = prefs.getBoolPref("define.tab_nextto");
			}
			else {
				prefs.setBoolPref("define.tab_nextto", true);
				arr["tab_nextto"] = true;
			}
			
			return arr;
		}
		catch (e) {
			alert("Preferences error\n" + e.name + ": " + e.message);
			return arr;
		}
	},
	
	
	showPrefsValues: function() {
		var settings = this.loadSettings();
		var elem = document.getElementById("define_target")
		if (elem != null) {
			elem.selectedIndex = settings["target"];
		}
		
		elem = document.getElementById("define_focus_tab")
		if (elem != null) {
			elem.checked = settings["focus_tab"];
		}
		
		elem = document.getElementById("define_tab_nextto")
		if (elem != null) {
			elem.checked = settings["tab_nextto"];
		}
		
		this.toggleOpsTarget();
	},
	
	
	saveSettings: function() {
		try {
			var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
			var elem = document.getElementById("define_target")
			if (elem != null) {
				prefs.setIntPref("define.target", elem.selectedIndex);
			}
			
			elem = document.getElementById("define_focus_tab")
			if (elem != null) {
				prefs.setBoolPref("define.focus_tab", elem.checked);
			}
			
			elem = document.getElementById("define_tab_nextto")
			if (elem != null) {
				prefs.setBoolPref("define.tab_nextto", elem.checked);
			}
			
			window.close();
		}
		catch (e) {
			alert("Preferences error\n" + e.name + ": " + e.message);
		}
	},
	
	
	toggleOpsTarget: function() {
		var elem = document.getElementById("define_target");
		var disabled = (elem.selectedIndex != 1);
		elem = document.getElementById("define_focus_tab");
		elem.disabled = disabled;
		elem = document.getElementById("define_tab_nextto");
		elem.disabled = disabled;
	}
};


window.addEventListener("load", define_extension.showContextMenuEvent, true);